/* ------------------------------------------------------------------------
System       : cc65 cross-development environment
Platform     : Apple IIgs or Apple IIe with VOC or CB

Bill Buckels
July 2014

Licence      : You may use this program for whatever you wish as long
			   as you agree that neither Charlie nor Bill have any
			   warranty or liability obligations whatsoever from said
			   use.
------------------------------------------------------------------------ */
#include <peekpoke.h>

#define  gswitch ((unsigned char*)0xC029)
#define  shadow  ((unsigned char*)0xC035)

/* graphics - save previous setting of gswitch */
unsigned char gsave = 0xff;
/* shadow memory - save previous setting of shadow switch */
unsigned char ssave = 0xff;

/* move a block of data from main to auxiliary memory */
#pragma optimize (push,off)
void maintoaux(unsigned src0, unsigned src1, unsigned dest0)
{

    unsigned *src  = (unsigned *)0x3c;
    unsigned *dest = (unsigned *)0x42;

    src[0] = src0;
    src[1] = src1;
    dest[0] = dest0;

    asm("sec");
    asm("jsr $c311");

}
#pragma optimize (pop)

/* move a block of data from auxiliary to main memory */
#pragma optimize (push,off)
void auxtomain(unsigned src0, unsigned src1, unsigned dest0)
{

    unsigned *src  = (unsigned *)0x3c;
    unsigned *dest = (unsigned *)0x42;

    src[0] = src0;
    src[1] = src1;
    dest[0] = dest0;

    asm("clc");
    asm("jsr $c311");

}
#pragma optimize (pop)

/* turn-on SHR */
void shgron(void)
{

    gsave = gswitch[0]; 		/* save previous gswitch settings */
	gswitch[0]= gsave | 0xc0;   /* shgr on  - set bits 6 and 7 */

	ssave = shadow[0];
	/* Bank $01 is shadowed into $E1 by clearing bit
       3 of the Shadow register at $C035 */
	shadow[0] = ssave & 0xf7;

}

/* turn-off SHR */
void shgroff()
{
	gswitch[0]=gsave; /* shgr off - restore previous setting */
	shadow[0] =ssave;
}

/* buffers for 16 palettes and 200 scanline control bytes */
unsigned char palbuf[512], scbbuf[256];

void clearpalette()
{
	unsigned src1 = (unsigned)&palbuf[0];
	unsigned src2 = src1 + 511;

	/* zero palette in auxiliary memory */
	maintoaux(src1,src2,0x9e00);
}

void clearscbs()
{
	unsigned src1 = (unsigned)&scbbuf[0];
	unsigned src2 = src1 + 199;

	/* zero scanline control bytes in auxiliary memory */
	maintoaux(src1,src2,0x9d00);

}

void initbuffers(void)
{
    memset((char *)&palbuf[0],0,512);
    memset((char *)&scbbuf[0],0,256);

}

/* turn-on text mode */
void texton(void)
{
	POKE(0xc051,0);  /* text */
	POKE(0xc054,0);  /* page 1 */
}
